<?php

namespace Drupal\Tests\periodic\Unit;

use Drupal\Component\Datetime\Time;
use Drupal\Core\State\State;
use Drupal\periodic\Event\PeriodicEvent;
use Drupal\periodic\PeriodicManager;
use Drupal\Tests\UnitTestCase;

/**
 * Test the Periodic Manager service.
 *
 * @group periodic
 *
 * @coversDefaultClass \Drupal\periodic\PeriodicManager
 */
class PeriodicManagerTest extends UnitTestCase {

  /**
   * Mock Time service.
   *
   * @var \Drupal\Component\Datetime\Time|\PHPUnit\Framework\MockObject\MockObject
   */
  private $time;

  /**
   * Mock State service.
   *
   * @var \Drupal\Core\State\State|\PHPUnit\Framework\MockObject\MockObject
   */
  private $state;

  /**
   * A Periodic Manager instance for testing.
   *
   * @var \Drupal\periodic\PeriodicManager
   */
  private $manager;

  /**
   * {@inheritdoc}
   */
  public function setUp(): void {
    parent::setUp();

    $this->time = $this->getMockBuilder(Time::class)
      ->disableOriginalConstructor()
      ->getMock();
    $this->time->expects($this->any())
      ->method('getRequestTime')
      ->willReturn(946684800);

    $this->state = $this->getMockBuilder(State::class)
      ->disableOriginalConstructor()
      ->getMock();

    $this->manager = new PeriodicManager($this->state, $this->time);
  }

  /**
   * A new task should be ready to run.
   */
  public function testReadyNewTask() {
    $this->state->expects($this->atLeastOnce())
      ->method('get')
      ->with('periodic.tasks', [])
      ->willReturn([]);
    $this->state->expects($this->never())
      ->method('set');

    $this->assertTrue($this->manager->ready('periodic.test', PeriodicEvent::INTERVAL_HOUR));
  }

  /**
   * A task with an elapsed interval should be ready.
   */
  public function testReadyStaleTask() {
    $this->state->expects($this->atLeastOnce())
      ->method('get')
      ->with('periodic.tasks', [])
      ->willReturn(['periodic.test' => 946681100]);
    $this->state->expects($this->never())
      ->method('set');

    $this->assertTrue($this->manager->ready('periodic.test', PeriodicEvent::INTERVAL_HOUR));
  }

  /**
   * A task should not be ready within its interval.
   */
  public function testReadyFreshTask() {
    $this->state->expects($this->atLeastOnce())
      ->method('get')
      ->with('periodic.tasks', [])
      ->willReturn(['periodic.test' => 946681500]);
    $this->state->expects($this->never())
      ->method('set');

    $this->assertFalse($this->manager->ready('periodic.test', PeriodicEvent::INTERVAL_HOUR));
  }

  /**
   * A new task should run immediately.
   */
  public function testExecuteNewTask() {
    $this->state->expects($this->atLeastOnce())
      ->method('get')
      ->with('periodic.tasks', [])
      ->willReturn([]);
    $this->state->expects($this->once())
      ->method('set')
      ->with('periodic.tasks', ['periodic.test' => 946684800]);

    $this->assertTrue($this->manager->execute('periodic.test', PeriodicEvent::INTERVAL_HOUR));
  }

  /**
   * A task with an elapsed interval should execute.
   */
  public function testExecuteStaleTask() {
    $this->state->expects($this->atLeastOnce())
      ->method('get')
      ->with('periodic.tasks', [])
      ->willReturn(['periodic.test' => 946681100]);
    $this->state->expects($this->once())
      ->method('set')
      ->with('periodic.tasks', ['periodic.test' => 946684800]);

    $this->assertTrue($this->manager->execute('periodic.test', PeriodicEvent::INTERVAL_HOUR));
  }

  /**
   * A task should not re-execute within its interval.
   */
  public function testExecuteFreshTask() {
    $this->state->expects($this->atLeastOnce())
      ->method('get')
      ->with('periodic.tasks', [])
      ->willReturn(['periodic.test' => 946681500]);
    $this->state->expects($this->never())
      ->method('set');

    $this->assertFalse($this->manager->execute('periodic.test', PeriodicEvent::INTERVAL_HOUR));
  }

}
